package com.sun.corba.se.impl.io;

import java.lang.reflect.Member;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.io.DataOutputStream;
import java.security.DigestOutputStream;
import java.security.MessageDigest;
import java.io.ByteArrayOutputStream;
import sun.misc.JavaSecurityAccess;
import java.security.AccessControlContext;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.UndeclaredThrowableException;
import sun.misc.SharedSecrets;
import java.io.InvalidClassException;
import com.sun.corba.se.impl.util.RepositoryId;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.security.AccessController;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.security.PrivilegedAction;
import java.util.HashSet;
import java.security.Permissions;
import java.lang.reflect.Proxy;
import org.omg.CORBA.ValueMember;
import java.io.Externalizable;
import java.util.Comparator;
import java.security.ProtectionDomain;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import sun.corba.Bridge;
import java.io.Serializable;

public class ObjectStreamClass implements Serializable
{
    private static final boolean DEBUG_SVUID = false;
    public static final long kDefaultUID = -1L;
    private static Object[] noArgsList;
    private static Class<?>[] noTypesList;
    private boolean isEnum;
    private static final Bridge bridge;
    private static final ObjectStreamClass.PersistentFieldsValue persistentFieldsValue;
    public static final int CLASS_MASK = 1553;
    public static final int FIELD_MASK = 223;
    public static final int METHOD_MASK = 3391;
    private static ObjectStreamClass.ObjectStreamClassEntry[] descriptorFor;
    private String name;
    private ObjectStreamClass superclass;
    private boolean serializable;
    private boolean externalizable;
    private ObjectStreamField[] fields;
    private Class<?> ofClass;
    boolean forProxyClass;
    private long suid;
    private String suidStr;
    private long actualSuid;
    private String actualSuidStr;
    int primBytes;
    int objFields;
    private boolean initialized;
    private Object lock;
    private boolean hasExternalizableBlockData;
    Method writeObjectMethod;
    Method readObjectMethod;
    private transient Method writeReplaceObjectMethod;
    private transient Method readResolveObjectMethod;
    private Constructor<?> cons;
    private transient ProtectionDomain[] domains;
    private String rmiiiopOptionalDataRepId;
    private ObjectStreamClass localClassDesc;
    private static Method hasStaticInitializerMethod;
    private static final long serialVersionUID = -6120832682080437368L;
    public static final ObjectStreamField[] NO_FIELDS;
    private static Comparator compareClassByName;
    private static final Comparator compareObjStrFieldsByName;
    private static Comparator compareMemberByName;
    
    static final ObjectStreamClass lookup(Class<?> clazz) {
        ObjectStreamClass lookupInternal = lookupInternal(clazz);
        if (lookupInternal.isSerializable() || lookupInternal.isExternalizable()) {
            return lookupInternal;
        }
        return null;
    }
    
    static ObjectStreamClass lookupInternal(Class<?> clazz) {
        Object descriptor = null;
        synchronized (ObjectStreamClass.descriptorFor) {
            descriptor = findDescriptorFor(clazz);
            if (descriptor == null) {
                int assignable = Serializable.class.isAssignableFrom(clazz);
                ObjectStreamClass lookup = null;
                if (!(assignable == 0)) {
                    Class<Object> superclass = clazz.getSuperclass();
                    if (superclass != null) {
                        lookup = lookup(superclass);
                    }
                }
                boolean n = false;
                if (!(assignable == 0)) {
                    n = ((lookup != null && lookup.isExternalizable()) || Externalizable.class.isAssignableFrom(clazz));
                    if (!(n == 0)) {
                        assignable = false;
                    }
                }
                descriptor = new ObjectStreamClass(clazz, lookup, (boolean)assignable, (boolean)n);
            }
            descriptor.init();
        }
        return descriptor;
    }
    
    public final String getName() {
        return this.name;
    }
    
    public static final long getSerialVersionUID(Class<?> clazz) {
        ObjectStreamClass lookup = lookup(clazz);
        if (lookup != null) {
            return lookup.getSerialVersionUID();
        }
        return 0L;
    }
    
    public final long getSerialVersionUID() {
        return this.suid;
    }
    
    public final String getSerialVersionUIDStr() {
        if (this.suidStr == null) {
            this.suidStr = Long.toHexString(this.suid).toUpperCase();
        }
        return this.suidStr;
    }
    
    public static final long getActualSerialVersionUID(Class<?> clazz) {
        ObjectStreamClass lookup = lookup(clazz);
        if (lookup != null) {
            return lookup.getActualSerialVersionUID();
        }
        return 0L;
    }
    
    public final long getActualSerialVersionUID() {
        return this.actualSuid;
    }
    
    public final String getActualSerialVersionUIDStr() {
        if (this.actualSuidStr == null) {
            this.actualSuidStr = Long.toHexString(this.actualSuid).toUpperCase();
        }
        return this.actualSuidStr;
    }
    
    public final Class<?> forClass() {
        return this.ofClass;
    }
    
    public ObjectStreamField[] getFields() {
        if (this.fields.length > 0) {
            ObjectStreamField[] array = new ObjectStreamField[this.fields.length];
            System.arraycopy(this.fields, 0, array, 0, this.fields.length);
            return array;
        }
        return this.fields;
    }
    
    public boolean hasField(ValueMember valueMember) {
        try {
            int i = 0;
            while (i < this.fields.length) {
                if (this.fields[i].getName().equals(valueMember.name) && this.fields[i].getSignature().equals(ValueUtility.getSignature(valueMember))) {
                    return true;
                }
                ++i;
            }
        }
        catch (Exception ex) {}
        return false;
    }
    
    final ObjectStreamField[] getFieldsNoCopy() {
        return this.fields;
    }
    
    public final ObjectStreamField getField(String s) {
        int i = this.fields.length - 1;
        while (i >= 0) {
            if (s.equals(this.fields[i].getName())) {
                return this.fields[i];
            }
            --i;
        }
        return null;
    }
    
    public Serializable writeReplace(Serializable obj) {
        if (this.writeReplaceObjectMethod != null) {
            try {
                return (Serializable)this.writeReplaceObjectMethod.invoke(obj, ObjectStreamClass.noArgsList);
            }
            catch (Throwable cause) {
                throw new RuntimeException(cause);
            }
        }
        else {
            return obj;
        }
    }
    
    public Object readResolve(Object obj) {
        if (this.readResolveObjectMethod != null) {
            try {
                return this.readResolveObjectMethod.invoke(obj, ObjectStreamClass.noArgsList);
            }
            catch (Throwable cause) {
                throw new RuntimeException(cause);
            }
        }
        else {
            return obj;
        }
    }
    
    @Override
    public final String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(this.name);
        sb.append(": static final long serialVersionUID = ");
        sb.append(Long.toString(this.suid));
        sb.append("L;");
        return sb.toString();
    }
    
    private ObjectStreamClass(Class<?> clazz, ObjectStreamClass superclass, boolean serializable, boolean externalizable) {
        this.suid = -1L;
        this.suidStr = null;
        this.actualSuid = -1L;
        this.actualSuidStr = null;
        this.initialized = false;
        this.lock = new Object();
        this.rmiiiopOptionalDataRepId = null;
        this.ofClass = clazz;
        if (Proxy.isProxyClass(clazz)) {
            this.forProxyClass = true;
        }
        this.name = clazz.getName();
        this.isEnum = Enum.class.isAssignableFrom(clazz);
        this.superclass = superclass;
        this.serializable = serializable;
        if (!this.forProxyClass) {
            this.externalizable = externalizable;
        }
        insertDescriptorFor(this);
    }
    
    private ProtectionDomain noPermissionsDomain() {
        Permissions permissions = new Permissions();
        permissions.setReadOnly();
        return new ProtectionDomain(null, permissions);
    }
    
    private ProtectionDomain[] getProtectionDomains(Constructor<?> constructor, Class<?> superclass) {
        ProtectionDomain[] array = null;
        if (constructor != null && superclass.getClassLoader() != null && System.getSecurityManager() != null) {
            Class<Object> declaringClass = constructor.getDeclaringClass();
            Object o = null;
            while (superclass != declaringClass) {
                ProtectionDomain protectionDomain = superclass.getProtectionDomain();
                if (protectionDomain != null) {
                    if (o == null) {
                        o = new HashSet();
                    }
                    o.add(protectionDomain);
                }
                superclass = superclass.getSuperclass();
                if (superclass == null) {
                    if (o == null) {
                        o = new HashSet();
                    }
                    else {
                        o.clear();
                    }
                    o.add(this.noPermissionsDomain());
                    break;
                }
                else {
                    continue;
                }
            }
            if (o != null) {
                array = (ProtectionDomain[])o.toArray(new ProtectionDomain[0]);
            }
        }
        return array;
    }
    
    private void init() {
        synchronized (this.lock) {
            if (this.initialized) {
                return;
            }
            final Class<Object> ofClass = this.ofClass;
            if (!this.serializable || this.externalizable || this.forProxyClass || this.name.equals("java.lang.String")) {
                this.fields = ObjectStreamClass.NO_FIELDS;
            }
            else if (this.serializable) {
                AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        ObjectStreamClass.this.fields = (ObjectStreamField[])ObjectStreamClass.persistentFieldsValue.get(ofClass);
                        if (ObjectStreamClass.this.fields == null) {
                            Field[] declaredFields = ofClass.getDeclaredFields();
                            int n = 0;
                            ObjectStreamField[] array = new ObjectStreamField[declaredFields.length];
                            for (int j = 0; j < declaredFields.length; ++j) {
                                Field field = declaredFields[j];
                                int modifiers = field.getModifiers();
                                if (!Modifier.isStatic(modifiers) && !Modifier.isTransient(modifiers)) {
                                    field.setAccessible(true);
                                    array[n++] = new ObjectStreamField(field);
                                }
                            }
                            ObjectStreamClass.this.fields = new ObjectStreamField[n];
                            System.arraycopy(array, 0, ObjectStreamClass.this.fields, 0, n);
                        }
                        else {
                            for (int i = ObjectStreamClass.this.fields.length - 1; i >= 0; --i) {
                                try {
                                    Field declaredField = ofClass.getDeclaredField(ObjectStreamClass.this.fields[i].getName());
                                    if (ObjectStreamClass.this.fields[i].getType() == declaredField.getType()) {
                                        declaredField.setAccessible(true);
                                        ObjectStreamClass.this.fields[i].setField(declaredField);
                                    }
                                }
                                catch (NoSuchFieldException ex) {}
                            }
                        }
                        return null;
                    }
                });
                if (this.fields.length > 1) {
                    Arrays.sort(this.fields);
                }
                this.computeFieldInfo();
            }
            if (this.isNonSerializable() || this.isEnum) {
                this.suid = 0L;
            }
            else {
                AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        if (ObjectStreamClass.this.forProxyClass) {
                            ObjectStreamClass.this.suid = 0L;
                        }
                        else {
                            try {
                                Field declaredField = ofClass.getDeclaredField("serialVersionUID");
                                int modifiers = declaredField.getModifiers();
                                if (Modifier.isStatic(modifiers) && Modifier.isFinal(modifiers)) {
                                    declaredField.setAccessible(true);
                                    ObjectStreamClass.this.suid = declaredField.getLong(ofClass);
                                }
                                else {
                                    ObjectStreamClass.this.suid = ObjectStreamClass.access$300(ofClass);
                                }
                            }
                            catch (NoSuchFieldException ex) {
                                ObjectStreamClass.this.suid = ObjectStreamClass.access$300(ofClass);
                            }
                            catch (IllegalAccessException ex2) {
                                ObjectStreamClass.this.suid = ObjectStreamClass.access$300(ofClass);
                            }
                        }
                        ObjectStreamClass.this.writeReplaceObjectMethod = ObjectStreamClass.access$600(ofClass, "writeReplace", ObjectStreamClass.noTypesList, Object.class);
                        ObjectStreamClass.this.readResolveObjectMethod = ObjectStreamClass.access$600(ofClass, "readResolve", ObjectStreamClass.noTypesList, Object.class);
                        ObjectStreamClass.this.domains = new ProtectionDomain[] { ObjectStreamClass.access$900(ObjectStreamClass.this) };
                        if (ObjectStreamClass.this.externalizable) {
                            ObjectStreamClass.this.cons = ObjectStreamClass.access$1200(ofClass);
                        }
                        else {
                            ObjectStreamClass.this.cons = ObjectStreamClass.access$1300(ofClass);
                        }
                        ObjectStreamClass.this.domains = ObjectStreamClass.access$1400(ObjectStreamClass.this, ObjectStreamClass.this.cons, ofClass);
                        if (ObjectStreamClass.this.serializable && !ObjectStreamClass.this.forProxyClass) {
                            ObjectStreamClass.this.writeObjectMethod = ObjectStreamClass.access$1600(ofClass, "writeObject", new Class[] { ObjectOutputStream.class }, Void.TYPE);
                            ObjectStreamClass.this.readObjectMethod = ObjectStreamClass.access$1600(ofClass, "readObject", new Class[] { ObjectInputStream.class }, Void.TYPE);
                        }
                        return null;
                    }
                });
            }
            this.actualSuid = computeStructuralUID(this, ofClass);
            if (this.hasWriteObject()) {
                this.rmiiiopOptionalDataRepId = this.computeRMIIIOPOptionalDataRepId();
            }
            this.initialized = true;
        }
    }
    
    private static Method getPrivateMethod(Class<?> clazz, String name, Class<?>[] parameterTypes, Class<?> clazz2) {
        try {
            Method declaredMethod = clazz.getDeclaredMethod(name, parameterTypes);
            declaredMethod.setAccessible(true);
            int modifiers = declaredMethod.getModifiers();
            return (declaredMethod.getReturnType() == clazz2 && (modifiers & 0x8) == 0x0 && (modifiers & 0x2) != 0x0) ? declaredMethod : null;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    private String computeRMIIIOPOptionalDataRepId() {
        StringBuffer sb = new StringBuffer("RMI:org.omg.custom.");
        sb.append(RepositoryId.convertToISOLatin1(this.getName()));
        sb.append(':');
        sb.append(this.getActualSerialVersionUIDStr());
        sb.append(':');
        sb.append(this.getSerialVersionUIDStr());
        return sb.toString();
    }
    
    public final String getRMIIIOPOptionalDataRepId() {
        return this.rmiiiopOptionalDataRepId;
    }
    
    ObjectStreamClass(String name, long suid) {
        this.suid = -1L;
        this.suidStr = null;
        this.actualSuid = -1L;
        this.actualSuidStr = null;
        this.initialized = false;
        this.lock = new Object();
        this.rmiiiopOptionalDataRepId = null;
        this.name = name;
        this.suid = suid;
        this.superclass = null;
    }
    
    final void setClass(Class<?> ofClass) throws InvalidClassException {
        if (ofClass == null) {
            this.localClassDesc = null;
            this.ofClass = null;
            this.computeFieldInfo();
            return;
        }
        this.localClassDesc = lookupInternal(ofClass);
        if (this.localClassDesc == null) {
            throw new InvalidClassException(ofClass.getName(), "Local class not compatible");
        }
        if (this.suid != this.localClassDesc.suid) {
            int n = this.isNonSerializable() || this.localClassDesc.isNonSerializable();
            if ((!ofClass.isArray() || ofClass.getName().equals(this.name)) && n == 0) {
                throw new InvalidClassException(ofClass.getName(), "Local class not compatible: stream classdesc serialVersionUID=" + this.suid + " local class serialVersionUID=" + this.localClassDesc.suid);
            }
        }
        if (!compareClassNames(this.name, ofClass.getName(), '.')) {
            throw new InvalidClassException(ofClass.getName(), "Incompatible local class name. Expected class name compatible with " + this.name);
        }
        if (this.serializable != this.localClassDesc.serializable || this.externalizable != this.localClassDesc.externalizable || (!this.serializable && !this.externalizable)) {
            throw new InvalidClassException(ofClass.getName(), "Serialization incompatible with Externalization");
        }
        ObjectStreamField[] array = (ObjectStreamField[])this.localClassDesc.fields;
        ObjectStreamField[] array2 = (ObjectStreamField[])this.fields;
        int i = 0;
        for (int j = 0; j < array2.length; ++j) {
            while (i < array.length) {
                if (array2[j].getName().equals(array[i].getName())) {
                    if (array2[j].isPrimitive() && !array2[j].typeEquals(array[i])) {
                        throw new InvalidClassException(ofClass.getName(), "The type of field " + array2[j].getName() + " of class " + this.name + " is incompatible.");
                    }
                    array2[j].setField(array[i].getField());
                    break;
                }
                else {
                    ++i;
                }
            }
        }
        this.computeFieldInfo();
        this.ofClass = ofClass;
        this.readObjectMethod = this.localClassDesc.readObjectMethod;
        this.readResolveObjectMethod = this.localClassDesc.readResolveObjectMethod;
        return;
    }
    
    static boolean compareClassNames(String s, String other, char c) {
        int lastIndex = s.lastIndexOf((int)c);
        if (lastIndex < 0) {
            lastIndex = 0;
        }
        int lastIndex2 = other.lastIndexOf((int)c);
        if (lastIndex2 < 0) {
            lastIndex2 = 0;
        }
        return s.regionMatches(false, lastIndex, other, lastIndex2, s.length() - lastIndex);
    }
    
    final boolean typeEquals(ObjectStreamClass objectStreamClass) {
        return this.suid == objectStreamClass.suid && compareClassNames(this.name, objectStreamClass.name, '.');
    }
    
    final void setSuperclass(ObjectStreamClass superclass) {
        this.superclass = superclass;
    }
    
    final ObjectStreamClass getSuperclass() {
        return this.superclass;
    }
    
    final boolean hasReadObject() {
        return this.readObjectMethod != null;
    }
    
    final boolean hasWriteObject() {
        return this.writeObjectMethod != null;
    }
    
    final boolean isCustomMarshaled() {
        return this.hasWriteObject() || this.isExternalizable() || (this.superclass != null && this.superclass.isCustomMarshaled());
    }
    
    boolean hasExternalizableBlockDataMode() {
        return this.hasExternalizableBlockData;
    }
    
    Object newInstance() throws InstantiationException, InvocationTargetException, UnsupportedOperationException {
        if (!this.initialized) {
            throw new InternalError("Unexpected call when not initialized");
        }
        if (this.cons != null) {
            try {
                if (this.domains == null || this.domains.length == 0) {
                    return this.cons.newInstance(new Object[0]);
                }
                JavaSecurityAccess javaSecurityAccess = SharedSecrets.getJavaSecurityAccess();
                PrivilegedAction privilegedAction = new PrivilegedAction() {
                    @Override
                    public Object run() {
                        try {
                            return ObjectStreamClass.this.cons.newInstance(new Object[0]);
                        }
                        catch (InstantiationException | InvocationTargetException | IllegalAccessException undeclaredThrowable) {
                            throw new UndeclaredThrowableException(undeclaredThrowable);
                        }
                    }
                };
                try {
                    return javaSecurityAccess.doIntersectionPrivilege(privilegedAction, AccessController.getContext(), new AccessControlContext(this.domains));
                }
                catch (UndeclaredThrowableException ex) {
                    Throwable cause = ex.getCause();
                    if ((cause instanceof InstantiationException)) {
                        throw (InstantiationException)cause;
                    }
                    if ((cause instanceof InvocationTargetException)) {
                        throw (InvocationTargetException)cause;
                    }
                    if ((cause instanceof IllegalAccessException)) {
                        throw (IllegalAccessException)cause;
                    }
                    throw ex;
                }
            }
            catch (IllegalAccessException cause2) {
                InternalError o = new InternalError();
                o.initCause(cause2);
                throw o;
            }
        }
        throw new UnsupportedOperationException();
    }
    
    private static Constructor getExternalizableConstructor(Class<?> clazz) {
        try {
            Constructor<Object> declaredConstructor = clazz.getDeclaredConstructor(new Class[0]);
            declaredConstructor.setAccessible(true);
            return ((declaredConstructor.getModifiers() & 0x1) != 0x0) ? declaredConstructor : null;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    private static Constructor getSerializableConstructor(Class<?> superclass) {
        while (Serializable.class.isAssignableFrom(superclass)) {
            if ((superclass = superclass.getSuperclass()) != null) {
                continue;
            }
            return null;
        }
        try {
            Constructor<Object> declaredConstructor = superclass.getDeclaredConstructor(new Class[0]);
            int modifiers = declaredConstructor.getModifiers();
            if ((modifiers & 0x2) != 0x0 || ((modifiers & 0x5) == 0x0 && !packageEquals(superclass, superclass))) {
                return null;
            }
            Constructor<Object> constructorForSerialization = ObjectStreamClass.bridge.newConstructorForSerialization(superclass, declaredConstructor);
            constructorForSerialization.setAccessible(true);
            return constructorForSerialization;
        }
        catch (NoSuchMethodException ex) {
            return null;
        }
    }
    
    final ObjectStreamClass localClassDescriptor() {
        return this.localClassDesc;
    }
    
    boolean isSerializable() {
        return this.serializable;
    }
    
    boolean isExternalizable() {
        return this.externalizable;
    }
    
    boolean isNonSerializable() {
        return !this.externalizable && !this.serializable;
    }
    
    private void computeFieldInfo() {
        this.primBytes = 0;
        this.objFields = 0;
        for (int i = 0; i < this.fields.length; ++i) {
            switch (this.fields[i].getTypeCode()) {
                case 'B':
                case 'Z': {
                    ++this.primBytes;
                    break;
                }
                case 'C':
                case 'S': {
                    this.primBytes += 2;
                    break;
                }
                case 'F':
                case 'I': {
                    this.primBytes += 4;
                    break;
                }
                case 'D':
                case 'J': {
                    this.primBytes += 8;
                    break;
                }
                case 'L':
                case '[': {
                    ++this.objFields;
                    break;
                }
            }
        }
    }
    
    private static void msg(String x) {
        System.out.println(x);
    }
    
    private static long _computeSerialVersionUID(Class<?> clazz) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream(512);
        long n = 0L;
        try {
            MessageDigest instance = MessageDigest.getInstance("SHA");
            DataOutputStream dataOutputStream = new DataOutputStream(new DigestOutputStream(stream, instance));
            dataOutputStream.writeUTF(clazz.getName());
            int n2 = clazz.getModifiers() & 0x611;
            Method[] declaredMethods = clazz.getDeclaredMethods();
            if ((n2 & 0x200) != 0x0) {
                n2 &= 0xFFFFFBFF;
                if (declaredMethods.length > 0) {
                    n2 |= 0x400;
                }
            }
            dataOutputStream.writeInt(n2 & 0x611);
            if (!clazz.isArray()) {
                Class[] interfaces = clazz.getInterfaces();
                Arrays.sort(interfaces, ObjectStreamClass.compareClassByName);
                for (int i = 0; i < interfaces.length; ++i) {
                    dataOutputStream.writeUTF(interfaces[i].getName());
                }
            }
            Field[] declaredFields = clazz.getDeclaredFields();
            Arrays.sort(declaredFields, ObjectStreamClass.compareMemberByName);
            for (int j = 0; j < declaredFields.length; ++j) {
                Field field = declaredFields[j];
                int modifiers = field.getModifiers();
                if (Modifier.isPrivate(modifiers)) {
                    if (!Modifier.isTransient(modifiers)) {
                        if (Modifier.isStatic(modifiers)) {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                dataOutputStream.writeUTF(field.getName());
                dataOutputStream.writeInt(modifiers & 0xDF);
                dataOutputStream.writeUTF(getSignature(field.getType()));
            }
            if (hasStaticInitializer(clazz)) {
                dataOutputStream.writeUTF("<clinit>");
                dataOutputStream.writeInt(8);
                dataOutputStream.writeUTF("()V");
            }
            ObjectStreamClass.MethodSignature[] removePrivateAndSort = ObjectStreamClass.MethodSignature.removePrivateAndSort(clazz.getDeclaredConstructors());
            for (int k = 0; k < removePrivateAndSort.length; ++k) {
                ObjectStreamClass.MethodSignature methodSignature = removePrivateAndSort[k];
                String str = "<init>";
                String replace = methodSignature.signature.replace('/', '.');
                dataOutputStream.writeUTF(str);
                dataOutputStream.writeInt(methodSignature.member.getModifiers() & 0xD3F);
                dataOutputStream.writeUTF(replace);
            }
            ObjectStreamClass.MethodSignature[] removePrivateAndSort2 = ObjectStreamClass.MethodSignature.removePrivateAndSort(declaredMethods);
            for (int l = 0; l < removePrivateAndSort2.length; ++l) {
                ObjectStreamClass.MethodSignature methodSignature2 = removePrivateAndSort2[l];
                String replace2 = methodSignature2.signature.replace('/', '.');
                dataOutputStream.writeUTF(methodSignature2.member.getName());
                dataOutputStream.writeInt(methodSignature2.member.getModifiers() & 0xD3F);
                dataOutputStream.writeUTF(replace2);
            }
            dataOutputStream.flush();
            byte[] digest = instance.digest();
            for (int n3 = 0; n3 < Math.min(8, digest.length); ++n3) {
                n += (long)(digest[n3] & 0xFF) << n3 * 8;
            }
        }
        catch (IOException ex) {
            n = -1L;
        }
        catch (NoSuchAlgorithmException cause) {
            SecurityException ex2 = new SecurityException();
            ex2.initCause(cause);
            throw ex2;
        }
        return n;
    }
    
    private static long computeStructuralUID(ObjectStreamClass objectStreamClass, Class<?> clazz) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream(512);
        long n = 0L;
        try {
            if (!Serializable.class.isAssignableFrom(clazz) || clazz.isInterface()) {
                return 0L;
            }
            if (Externalizable.class.isAssignableFrom(clazz)) {
                return 1L;
            }
            MessageDigest instance = MessageDigest.getInstance("SHA");
            DataOutputStream dataOutputStream = new DataOutputStream(new DigestOutputStream(stream, instance));
            Class<Object> superclass = clazz.getSuperclass();
            if (superclass != null) {
                dataOutputStream.writeLong(computeStructuralUID(lookup(superclass), superclass));
            }
            if (objectStreamClass.hasWriteObject()) {
                dataOutputStream.writeInt(2);
            }
            else {
                dataOutputStream.writeInt(1);
            }
            ObjectStreamField[] fields = objectStreamClass.getFields();
            if (fields.length > 1) {
                Arrays.sort(fields, ObjectStreamClass.compareObjStrFieldsByName);
            }
            for (int i = 0; i < fields.length; ++i) {
                dataOutputStream.writeUTF(fields[i].getName());
                dataOutputStream.writeUTF(fields[i].getSignature());
            }
            dataOutputStream.flush();
            byte[] digest = instance.digest();
            for (int j = 0; j < Math.min(8, digest.length); ++j) {
                n += (long)(digest[j] & 0xFF) << j * 8;
            }
        }
        catch (IOException ex) {
            n = -1L;
        }
        catch (NoSuchAlgorithmException cause) {
            SecurityException ex2 = new SecurityException();
            ex2.initCause(cause);
            throw ex2;
        }
        return n;
    }
    
    static String getSignature(Class<?> componentType) {
        String s = null;
        if (componentType.isArray()) {
            int n = 0;
            while (componentType.isArray()) {
                ++n;
                componentType = componentType.getComponentType();
            }
            StringBuffer o = new StringBuffer();
            for (int i = 0; i < n; ++i) {
                o.append("[");
            }
            o.append(getSignature(componentType));
            s = o.toString();
        }
        else if (componentType.isPrimitive()) {
            if (componentType == Integer.TYPE) {
                s = "I";
            }
            else if (componentType == Byte.TYPE) {
                s = "B";
            }
            else if (componentType == Long.TYPE) {
                s = "J";
            }
            else if (componentType == Float.TYPE) {
                s = "F";
            }
            else if (componentType == Double.TYPE) {
                s = "D";
            }
            else if (componentType == Short.TYPE) {
                s = "S";
            }
            else if (componentType == Character.TYPE) {
                s = "C";
            }
            else if (componentType == Boolean.TYPE) {
                s = "Z";
            }
            else if (componentType == Void.TYPE) {
                s = "V";
            }
        }
        else {
            s = "L" + componentType.getName().replace('.', '/') + ";";
        }
        return s;
    }
    
    static String getSignature(Method method) {
        StringBuffer sb = new StringBuffer();
        sb.append("(");
        Class[] parameterTypes = method.getParameterTypes();
        for (int i = 0; i < parameterTypes.length; ++i) {
            sb.append(getSignature(parameterTypes[i]));
        }
        sb.append(")");
        sb.append(getSignature(method.getReturnType()));
        return sb.toString();
    }
    
    static String getSignature(Constructor constructor) {
        StringBuffer sb = new StringBuffer();
        sb.append("(");
        Class[] parameterTypes = constructor.getParameterTypes();
        for (int i = 0; i < parameterTypes.length; ++i) {
            sb.append(getSignature(parameterTypes[i]));
        }
        sb.append(")V");
        return sb.toString();
    }
    
    private static ObjectStreamClass findDescriptorFor(Class<?> clazz) {
        int n = (clazz.hashCode() & Integer.MAX_VALUE) % ObjectStreamClass.descriptorFor.length;
        ObjectStreamClass.ObjectStreamClassEntry next;
        while ((next = ObjectStreamClass.descriptorFor[n]) != null && next.get() == null) {
            ObjectStreamClass.descriptorFor[n] = next.next;
        }
        for (; next != null; next = next.next) {
            ObjectStreamClass objectStreamClass = (ObjectStreamClass)(ObjectStreamClass)next.get();
            if (objectStreamClass == null) {
            }
            else {
                if (objectStreamClass.ofClass == clazz) {
                    return objectStreamClass;
                }
            }
        }
        return null;
    }
    
    private static void insertDescriptorFor(ObjectStreamClass objectStreamClass) {
        if (findDescriptorFor(objectStreamClass.ofClass) != null) {
            return;
        }
        int n = (objectStreamClass.ofClass.hashCode() & Integer.MAX_VALUE) % ObjectStreamClass.descriptorFor.length;
        ObjectStreamClass.ObjectStreamClassEntry objectStreamClassEntry = new ObjectStreamClassEntry(objectStreamClass);
        objectStreamClassEntry.next = ObjectStreamClass.descriptorFor[n];
        ObjectStreamClass.descriptorFor[n] = objectStreamClassEntry;
        return;
    }
    
    private static Field[] getDeclaredFields(final Class<?> clazz) {
        return (Field[])(Field[])AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return clazz.getDeclaredFields();
            }
        });
    }
    
    private static boolean hasStaticInitializer(Class<?> clazz) {
        if (ObjectStreamClass.hasStaticInitializerMethod == null) {
            Class<Object> clazz2 = null;
            try {
                if (clazz2 == null) {
                    clazz2 = java.io.ObjectStreamClass.class;
                }
                ObjectStreamClass.hasStaticInitializerMethod = clazz2.getDeclaredMethod("hasStaticInitializer", new Class[] { Class.class });
            }
            catch (NoSuchMethodException ex) {}
            if (ObjectStreamClass.hasStaticInitializerMethod == null) {
                throw new InternalError("Can't find hasStaticInitializer method on " + clazz2.getName());
            }
            ObjectStreamClass.hasStaticInitializerMethod.setAccessible(true);
        }
        try {
            return (boolean)ObjectStreamClass.hasStaticInitializerMethod.invoke(null, new Object[] { clazz });
        }
        catch (Exception cause) {
            InternalError internalError = new InternalError("Error invoking hasStaticInitializer");
            internalError.initCause(cause);
            throw internalError;
        }
    }
    
    private static Method getInheritableMethod(Class<?> superclass, String name, Class<?>[] parameterTypes, Class<?> clazz) {
        Method declaredMethod = null;
        while (superclass != null) {
            try {
                declaredMethod = superclass.getDeclaredMethod(name, parameterTypes);
                break;
            }
            catch (NoSuchMethodException ex) {
                superclass = superclass.getSuperclass();
            }
        }
        if (declaredMethod == null || declaredMethod.getReturnType() != clazz) {
            return null;
        }
        declaredMethod.setAccessible(true);
        int modifiers = declaredMethod.getModifiers();
        if ((modifiers & 0x408) != 0x0) {
            return null;
        }
        if ((modifiers & 0x5) != 0x0) {
            return declaredMethod;
        }
        if ((modifiers & 0x2) != 0x0) {
            return (superclass == superclass) ? declaredMethod : null;
        }
        return packageEquals(superclass, superclass) ? declaredMethod : null;
    }
    
    private static boolean packageEquals(Class<?> clazz, Class<?> clazz2) {
        Package package1 = clazz.getPackage();
        Package package2 = clazz2.getPackage();
        return package1 == package2 || (package1 != null && package1.equals(package2));
    }
    
    static long access$300(Class clazz) {
        return _computeSerialVersionUID(clazz);
    }
    
    static Method access$600(Class clazz, String s, Class[] array, Class clazz2) {
        return getInheritableMethod(clazz, s, array, clazz2);
    }
    
    static ProtectionDomain access$900(ObjectStreamClass objectStreamClass) {
        return objectStreamClass.noPermissionsDomain();
    }
    
    static Constructor access$1200(Class clazz) {
        return getExternalizableConstructor(clazz);
    }
    
    static Constructor access$1300(Class clazz) {
        return getSerializableConstructor(clazz);
    }
    
    static ProtectionDomain[] access$1400(ObjectStreamClass objectStreamClass, Constructor constructor, Class clazz) {
        return objectStreamClass.getProtectionDomains(constructor, clazz);
    }
    
    static Method access$1600(Class clazz, String s, Class[] array, Class clazz2) {
        return getPrivateMethod(clazz, s, array, clazz2);
    }
    
    static {
        ObjectStreamClass.noArgsList = new Object[0];
        ObjectStreamClass.noTypesList = new Class[0];
        bridge = (Bridge)AccessController.doPrivileged(new PrivilegedAction<Bridge>() {
            @Override
            public Bridge run() {
                return Bridge.get();
            }
        });
        persistentFieldsValue = new ObjectStreamClass.PersistentFieldsValue();
        ObjectStreamClass.descriptorFor = new ObjectStreamClass.ObjectStreamClassEntry[61];
        ObjectStreamClass.hasStaticInitializerMethod = null;
        NO_FIELDS = new ObjectStreamField[0];
        ObjectStreamClass.compareClassByName = new ObjectStreamClass.CompareClassByName();
        compareObjStrFieldsByName = new ObjectStreamClass.CompareObjStrFieldsByName();
        ObjectStreamClass.compareMemberByName = new ObjectStreamClass.CompareMemberByName();
    }
    
    private static final class PersistentFieldsValue extends ClassValue<ObjectStreamField[]>
    {
        PersistentFieldsValue() {
        }
        
        @Override
        protected ObjectStreamField[] computeValue(Class<?> obj) {
            try {
                Field declaredField = obj.getDeclaredField("serialPersistentFields");
                int modifiers = declaredField.getModifiers();
                if (Modifier.isPrivate(modifiers) && Modifier.isStatic(modifiers) && Modifier.isFinal(modifiers)) {
                    declaredField.setAccessible(true);
                    return translateFields((java.io.ObjectStreamField[])(java.io.ObjectStreamField[])declaredField.get(obj));
                }
            }
            catch (NoSuchFieldException | IllegalAccessException | IllegalArgumentException | ClassCastException ex) {}
            return null;
        }
        
        private static ObjectStreamField[] translateFields(java.io.ObjectStreamField[] array) {
            ObjectStreamField[] array2 = new ObjectStreamField[array.length];
            for (int i = 0; i < array.length; ++i) {
                array2[i] = new ObjectStreamField(array[i].getName(), array[i].getType());
            }
            return array2;
        }
    }
    
    private static class ObjectStreamClassEntry
    {
        ObjectStreamClass.ObjectStreamClassEntry next;
        private ObjectStreamClass c;
        
        ObjectStreamClassEntry(ObjectStreamClass c) {
            this.c = c;
        }
        
        public Object get() {
            return this.c;
        }
    }
    
    private static class CompareClassByName implements Comparator
    {
        @Override
        public int compare(Object o, Object o2) {
            return ((Class)o).getName().compareTo(((Class)o2).getName());
        }
    }
    
    private static class CompareObjStrFieldsByName implements Comparator
    {
        @Override
        public int compare(Object o, Object o2) {
            return ((ObjectStreamField)o).getName().compareTo(((ObjectStreamField)o2).getName());
        }
    }
    
    private static class CompareMemberByName implements Comparator
    {
        @Override
        public int compare(Object o, Object o2) {
            String s = ((Member)o).getName();
            String anotherString = ((Member)o2).getName();
            if ((o instanceof Method)) {
                s += ObjectStreamClass.getSignature((Method)o);
                anotherString += ObjectStreamClass.getSignature((Method)o2);
            }
            else if ((o instanceof Constructor)) {
                s += ObjectStreamClass.getSignature((Constructor)o);
                anotherString += ObjectStreamClass.getSignature((Constructor)o2);
            }
            return s.compareTo(anotherString);
        }
    }
    
    private static class MethodSignature implements Comparator
    {
        Member member;
        String signature;
        
        static ObjectStreamClass.MethodSignature[] removePrivateAndSort(Member[] array) {
            int n = 0;
            for (int i = 0; i < array.length; ++i) {
                if (!Modifier.isPrivate(array[i].getModifiers())) {
                    ++n;
                }
            }
            ObjectStreamClass.MethodSignature[] a = new ObjectStreamClass.MethodSignature[n];
            int n2 = 0;
            for (int j = 0; j < array.length; ++j) {
                if (!Modifier.isPrivate(array[j].getModifiers())) {
                    a[n2] = new ObjectStreamClass.MethodSignature(array[j]);
                    ++n2;
                }
            }
            if (n2 > 0) {
                Arrays.sort(a, a[0]);
                return a;
            }
            return a;
        }
        
        @Override
        public int compare(Object o, Object o2) {
            if (o == o2) {
                return 0;
            }
            ObjectStreamClass.MethodSignature methodSignature = (ObjectStreamClass.MethodSignature)o;
            ObjectStreamClass.MethodSignature methodSignature2 = (ObjectStreamClass.MethodSignature)o2;
            int n;
            if (this.isConstructor()) {
                n = methodSignature.signature.compareTo(methodSignature2.signature);
            }
            else {
                n = methodSignature.member.getName().compareTo(methodSignature2.member.getName());
                if (n == 0) {
                    n = methodSignature.signature.compareTo(methodSignature2.signature);
                }
            }
            return n;
        }
        
        private final boolean isConstructor() {
            return this.member instanceof Constructor;
        }
        
        private MethodSignature(Member member) {
            this.member = member;
            if (this.isConstructor()) {
                this.signature = ObjectStreamClass.getSignature((Constructor)member);
            }
            else {
                this.signature = ObjectStreamClass.getSignature((Method)member);
            }
        }
    }
}
